import 'dart:convert';
import 'dart:io';
import 'package:xml/xml.dart' as xml;

import '../model/services_android_model.dart';
import '../model/services_ios_model.dart';
import '../model/services_web_model.dart';
import '../utils/utils.dart';

class Setup {
  static Future<void> updateStringsXmlValues({
    String? appName,
    String? facebookAppId,
    String? facebookClientToken,
    String? admobAppId,
  }) async {
    final file = File('android/app/src/main/res/values/strings.xml');
    if (!await file.exists()) {
      print('strings.xml file does not exist');
      return;
    }

    final document = xml.XmlDocument.parse(await file.readAsString());
    final builder = xml.XmlBuilder()
      ..processing('xml', 'version="1.0" encoding="utf-8"');

    builder.element('resources', nest: () {
      document.findAllElements('string').forEach((element) {
        final name = element.getAttribute('name');
        var newText = element.innerText; // Default to existing text

        // Determine the new text based on the provided non-null parameters
        if (name == 'app_name' && appName != null) {
          newText = appName;
        } else if (name == 'facebook_app_id' && facebookAppId != null) {
          newText = facebookAppId;
        } else if (name == 'fb_login_protocol_scheme' &&
            facebookAppId != null) {
          newText = 'fb$facebookAppId';
        } else if (name == 'facebook_client_token' &&
            facebookClientToken != null) {
          newText = facebookClientToken;
        } else if (name == 'admob_app_id' && admobAppId != null) {
          newText = admobAppId;
        }

        // Update or keep the string element based on newText value
        if (newText.isNotEmpty) {
          // If newText is not null and not empty, update the element
          builder.element('string', nest: newText, attributes: {'name': name!});
        } else {
          // If newText is null or empty, keep the original XML structure
          builder.element('string', nest: () {
            builder.text(element.innerText);
          }, attributes: {'name': name!});
        }
      });
    });

    final newDocument = builder.buildDocument();
    await file
        .writeAsString(newDocument.toXmlString(pretty: true, indent: '    '));
    print('strings.xml updated successfully.');
  }

  static Future<void> updateInfoPlist({
    String? appName,
    String? facebookAppId,
    String? facebookClientToken,
    String? admobAppId,
    String? packageName,
    String? googleClientId,
    String? googleAppId,
  }) async {
    final file = File('ios/Runner/Info.plist');
    var contents = file.readAsStringSync();

    if (appName != null) {
      contents = contents.replaceAll(
          RegExp(r'<key>CFBundleName</key>\s*<string>.*?</string>'),
          '<key>CFBundleName</key>\n\t<string>$appName</string>');
      contents = contents.replaceAll(
          RegExp(r'<key>CFBundleDisplayName</key>\s*<string>.*?</string>'),
          '<key>CFBundleDisplayName</key>\n\t<string>$appName</string>');
      contents = contents.replaceAll(
          RegExp(r'<key>FacebookDisplayName</key>\s*<string>.*?</string>'),
          '<key>FacebookDisplayName</key>\n\t<string>$appName</string>');
    }

    if (facebookAppId != null) {
      contents = contents.replaceAll(
          RegExp(r'<key>FacebookAppID</key>\s*<string>.*?</string>'),
          '<key>FacebookAppID</key>\n\t<string>$facebookAppId</string>');
    }

    if (facebookClientToken != null) {
      contents = contents.replaceAll(
          RegExp(r'<key>FacebookClientToken</key>\s*<string>.*?</string>'),
          '<key>FacebookClientToken</key>\n\t<string>$facebookClientToken</string>');
    }

    if (admobAppId != null) {
      contents = contents.replaceAll(
          RegExp(r'<key>GADApplicationIdentifier</key>\s*<string>.*?</string>'),
          '<key>GADApplicationIdentifier</key>\n\t<string>$admobAppId</string>');
    }

    file.writeAsStringSync(contents);
    await continueUpdates(
      packageName: packageName,
      googleClientId: googleClientId,
      facebookAppId: facebookAppId,
      googleAppId: googleAppId,
    );
    print('Info.plist updated successfully.');
  }

  static Future<void> continueUpdates({
    String? packageName,
    String? googleClientId,
    String? facebookAppId,
    String? googleAppId,
  }) async {
    var path = 'ios/Runner/Info.plist';
    var file = File(path);

    var contents = await file.readAsString();
    var document = xml.XmlDocument.parse(contents);

    var urlTypes = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'CFBundleURLTypes')
        .nextElementSibling;

    for (var dict in urlTypes!.findElements('dict')) {
      var urlNameElement = dict.findElements('string').firstWhere((element) =>
          element.previousElementSibling!.innerText == 'CFBundleURLName');
      String? urlName = urlNameElement.innerText;

      switch (urlName) {
        case 'google_client_id':
          if(googleClientId != null){
            _updateSchemeValue(dict, googleClientId);
          }
          break;
        case 'facebook_id':
          if(facebookAppId != null){
            _updateSchemeValue(dict, 'fb$facebookAppId');
          }
          break;
        case 'google_app_id':
          if(googleAppId != null){
            _updateSchemeValue(dict, googleAppId);
          }
          break;
        case 'Bundle ID':
          if(packageName != null){
            _updateSchemeValue(dict, packageName);
          }
          break;
      }
    }

    await file.writeAsString(document.toXmlString(pretty: true, indent: '\t'));
  }

  static void _updateSchemeValue(xml.XmlElement dict, String newValue) {
    var schemesArray = dict
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'CFBundleURLSchemes')
        .nextElementSibling;

    schemesArray!.children.clear();
    var newSchemeString =
        xml.XmlElement(xml.XmlName('string'), [], [xml.XmlText(newValue)]);
    schemesArray.children.add(newSchemeString);
  }

  static Future<ServicesWebModel> extractGoogleServicesWebFile(String source) async {

    final file = File(source);

    // Read the file as a string
    String contents = await file.readAsString();

    // Regular expressions to extract data
    RegExp apiKeyPattern = RegExp(r'apiKey: "([^"]+)"');
    RegExp authDomainPattern = RegExp(r'authDomain: "([^"]+)"');
    RegExp projectIdPattern = RegExp(r'projectId: "([^"]+)"');
    RegExp storageBucketPattern = RegExp(r'storageBucket: "([^"]+)"');
    RegExp messagingSenderIdPattern = RegExp(r'messagingSenderId: "([^"]+)"');
    RegExp appIdPattern = RegExp(r'appId: "([^"]+)"');
    RegExp measurementIdPattern = RegExp(r'measurementId: "([^"]+)"');

    String apiKey = apiKeyPattern.firstMatch(contents)?.group(1) ?? '';
    String authDomain = authDomainPattern.firstMatch(contents)?.group(1) ?? '';
    String projectId = projectIdPattern.firstMatch(contents)?.group(1) ?? '';
    String storageBucket = storageBucketPattern.firstMatch(contents)?.group(1) ?? '';
    String messagingSenderId = messagingSenderIdPattern.firstMatch(contents)?.group(1) ?? '';
    String appId = appIdPattern.firstMatch(contents)?.group(1) ?? '';
    String measurementId = measurementIdPattern.firstMatch(contents)?.group(1) ?? '';

    return ServicesWebModel(
      appId: appId,
      projectId: projectId,
      apiKey: apiKey,
      storageBucket:storageBucket,
      senderId: messagingSenderId,
      authDomain: authDomain,
      measurementId: measurementId,
    );
  }

  static Future<ServicesAndroidModel> extractGoogleServicesAndroidFile(String source) async {
    final file = File(source);

    // Parse the string as JSON
    Map<String, dynamic> json = jsonDecode(await file.readAsString());

    // Extracting values
    String projectNumber = json['project_info']['project_number'];
    String projectId = json['project_info']['project_id'];
    String storageBucket = json['project_info']['storage_bucket'];
    String mobileSdkAppId = json['client'][0]['client_info']['mobilesdk_app_id'];
    String packageName = json['client'][0]['client_info']['android_client_info']['package_name'];
    List<String> clientIds = [];
    for (var client in json['client'][0]['oauth_client']) {
      clientIds.add(client['client_id']);
    }
    String currentKey = json['client'][0]['api_key'][0]['current_key'];

    return ServicesAndroidModel(
      appId: mobileSdkAppId,
      projectId: projectId,
      apiKey: currentKey,
      storageBucket: storageBucket,
      senderId: projectNumber,
      packageName: packageName,
      androidClientId: clientIds.first,
    );
  }

  static Future<ServicesIosModel> extractGoogleServicesIosFile(String source) async {
    final file = File(source);
    final document = xml.XmlDocument.parse(await file.readAsString());

    final reversedClientId = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'REVERSED_CLIENT_ID')
        .nextElementSibling!
        .innerText;

    final gcmSenderId = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'GCM_SENDER_ID')
        .nextElementSibling!
        .innerText;

    final appId = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'GOOGLE_APP_ID')
        .nextElementSibling!
        .innerText;

    final apiKey = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'API_KEY')
        .nextElementSibling!
        .innerText;

    final projectId = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'PROJECT_ID')
        .nextElementSibling!
        .innerText;

    final storageBucket = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'STORAGE_BUCKET')
        .nextElementSibling!
        .innerText;

    final iosClientId = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'CLIENT_ID')
        .nextElementSibling!
        .innerText;

    final iosBundleId = document
        .findAllElements('key')
        .firstWhere((element) => element.innerText == 'BUNDLE_ID')
        .nextElementSibling!
        .innerText;

    return ServicesIosModel(
        appId: appId,
        projectId: projectId,
        apiKey: apiKey,
        storageBucket: storageBucket,
        reversedClientId: reversedClientId,
        senderId: gcmSenderId,
        iosBundleId: iosBundleId,
        iosClientId: iosClientId,
        googleCodedAppId: Utils.convertGoogleAppIdToCoded(appId),
    );
  }

  static Future<void> updateHtmlFile({
    required String appName,
    String? firebaseWebClient,
    required String apiKey,
    required String authDomain,
    required String projectId,
    required String storageBucket,
    required String messagingSenderId,
    required String appId,
    required String measurementId,
  }) async {
    final filePath = 'web/index.html'; // Adjust the path as necessary
    final file = File(filePath);

    if (!await file.exists()) {
      print('File does not exist.');
      return;
    }

    String content = await file.readAsString();
    final titleTagRegex = RegExp(r'<title>(.*?)</title>');
    final metaAppTitleRegex = RegExp(r'<meta name="apple-mobile-web-app-title" content="([^"]*)">');

    final firebaseWebClientRegex = RegExp(r'<meta name="google-signin-client_id" content="([^"]*)">');

    final apiKeyRegex = RegExp(r'apiKey: "([^"]+)"');
    final authDomainRegex = RegExp(r'authDomain: "([^"]+)"');
    final projectIdRegex = RegExp(r'projectId: "([^"]+)"');
    final storageBucketRegex = RegExp(r'storageBucket: "([^"]+)"');
    final messagingSenderIdRegex = RegExp(r'messagingSenderId: "([^"]+)"');
    final appIdRegex = RegExp(r'appId: "([^"]+)"');
    final measurementIdRegex = RegExp(r'measurementId: "([^"]+)"');

    content = content.replaceAllMapped(titleTagRegex, (match) => '<title>$appName</title>');
    content = content.replaceAllMapped(metaAppTitleRegex, (match) => '<meta name="apple-mobile-web-app-title" content="$appName">');

    if(firebaseWebClient != null){
      content = content.replaceAllMapped(firebaseWebClientRegex, (match) => '<meta name="google-signin-client_id" content="$firebaseWebClient">');
    }

    content = content.replaceAllMapped(apiKeyRegex, (match) => 'apiKey: "$apiKey"');
    content = content.replaceAllMapped(authDomainRegex, (match) => 'authDomain: "$authDomain"');
    content = content.replaceAllMapped(projectIdRegex, (match) => 'projectId: "$projectId"');
    content = content.replaceAllMapped(storageBucketRegex, (match) => 'storageBucket: "$storageBucket"');
    content = content.replaceAllMapped(messagingSenderIdRegex, (match) => 'messagingSenderId: "$messagingSenderId"');
    content = content.replaceAllMapped(appIdRegex, (match) => 'appId: "$appId"');
    content = content.replaceAllMapped(measurementIdRegex, (match) => 'measurementId: "$measurementId"');


    await file.writeAsString(content);
    print('index.html updated successfully.');
  }

  static Future<void> updateFirebaseJsFile({
    required String apiKey,
    required String authDomain,
    required String projectId,
    required String storageBucket,
    required String messagingSenderId,
    required String appId,
    required String measurementId,
  }) async {
    final filePath = 'web/firebase-messaging-sw.js'; // Adjust the path as necessary
    final file = File(filePath);

    if (!await file.exists()) {
      print('File does not exist.');
      return;
    }

    String content = await file.readAsString();

    final apiKeyRegex = RegExp(r'apiKey: "([^"]+)"');
    final authDomainRegex = RegExp(r'authDomain: "([^"]+)"');
    final projectIdRegex = RegExp(r'projectId: "([^"]+)"');
    final storageBucketRegex = RegExp(r'storageBucket: "([^"]+)"');
    final messagingSenderIdRegex = RegExp(r'messagingSenderId: "([^"]+)"');
    final appIdRegex = RegExp(r'appId: "([^"]+)"');
    final measurementIdRegex = RegExp(r'measurementId: "([^"]+)"');

    content = content.replaceAllMapped(apiKeyRegex, (match) => 'apiKey: "$apiKey"');
    content = content.replaceAllMapped(authDomainRegex, (match) => 'authDomain: "$authDomain"');
    content = content.replaceAllMapped(projectIdRegex, (match) => 'projectId: "$projectId"');
    content = content.replaceAllMapped(storageBucketRegex, (match) => 'storageBucket: "$storageBucket"');
    content = content.replaceAllMapped(messagingSenderIdRegex, (match) => 'messagingSenderId: "$messagingSenderId"');
    content = content.replaceAllMapped(appIdRegex, (match) => 'appId: "$appId"');
    content = content.replaceAllMapped(measurementIdRegex, (match) => 'measurementId: "$measurementId"');


    await file.writeAsString(content);
    print('firebase-messaging-sw.js updated successfully.');
  }

  static Future<void> updateAppConfigFile(
      {String? appName,
        required String pushGcm,
        required String purchaseCode,
        required String validationKey,
        required String facebookAppId,
      }) async {

    final file = File('lib/app/app.dart');
    if (!await file.exists()) {
      print('File does not exist.');
      return;
    }

    String content = await file.readAsString();
    final appNamePattern = RegExp(r'static String appName = "([^"]+)";');
    final validationKeyPattern = RegExp(r'static String validationKey = "([^"]+)";');
    final purchaseCodePattern = RegExp(r'static String purchaseCode = "([^"]+)";');
    final pushGcmPattern = RegExp(r'static String pushGcm = "([^"]+)";');
    final facebookAppIdPattern = RegExp(r'static String facebookAppId = "([^"]+)";');

    content = content.replaceAll(validationKeyPattern, 'static String validationKey = "$validationKey";');
    content = content.replaceAll(purchaseCodePattern, 'static String purchaseCode = "$purchaseCode";');
    content = content.replaceAll(pushGcmPattern, 'static String pushGcm = "$pushGcm";');
    content = content.replaceAll(facebookAppIdPattern, 'static String facebookAppId = "$facebookAppId";');

    if(appName != null){
      content = content.replaceAll(appNamePattern, 'static String appName = "$appName";');
    }

    await file.writeAsString(content);
    print('App config updated successfully.');
  }

  static Future<void> updateFirebaseConfigWebFile({
        required String apiKey,
        required String appId,
        required String senderId,
        required String projectId,
        required String storageBucket,
        required String authDomain,
        required String measurementId,
      }) async {

    final file = File('lib/app/firebase.dart');
    if (!await file.exists()) {
      print('File does not exist.');
      return;
    }

    String content = await file.readAsString();

    final apiKeyPattern = RegExp(r'static const String webApiKey = "([^"]+)";');
    final appIdPattern = RegExp(r'static const String webAppId = "([^"]+)";');
    final senderIdPattern = RegExp(r'static const String webMessagingSenderId = "([^"]+)";');
    final projectIdPattern = RegExp(r'static const String webProjectId = "([^"]+)";');
    final storageBucketPattern = RegExp(r'static const String webStorageBucket = "([^"]+)";');
    final authDomainPattern = RegExp(r'static const String webAuthDomain = "([^"]+)";');
    final measurementIdPattern = RegExp(r'static const String webMeasurementId = "([^"]+)";');

    content = content.replaceAll(apiKeyPattern, 'static const String webApiKey = "$apiKey";');
    content = content.replaceAll(appIdPattern, 'static const String webAppId = "$appId";');
    content = content.replaceAll(senderIdPattern, 'static const String webMessagingSenderId = "$senderId";');
    content = content.replaceAll(projectIdPattern, 'static const String webProjectId = "$projectId";');
    content = content.replaceAll(storageBucketPattern, 'static const String webStorageBucket = "$storageBucket";');
    content = content.replaceAll(authDomainPattern, 'static const String webAuthDomain = "$authDomain";');
    content = content.replaceAll(measurementIdPattern, 'static const String webMeasurementId = "$measurementId";');

    await file.writeAsString(content);
    print('Firebase flutter web config updated successfully.');
  }
}
